/* Zodiac - DisplayManager.cp
 */

#include "DisplayManager.h"

DisplayManager::DisplayManager()
{
	m_surface = NULL;
}

DisplayManager::~DisplayManager()
{
	closeDisplay();
}

bool DisplayManager::openDisplay(int width, int height, int bitDepth, bool fullscreen)
{
	Uint32 flags;
	SDL_Event e;
	
	// Let SDL do it's basic initialization stuff
	if(SDL_InitSubSystem(SDL_INIT_VIDEO) != 0)
	{
		return false;
	}

	switch(bitDepth)
	{
	case 32:
	case 24:
	default:
		SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 8);
		SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 8);
		SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 8);
		break;

	case 16:
		SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 5);
		SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 6);
		SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 5);
		break;

	case 15:
		SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 5);
		SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 5);
		SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 5);
		break;
	}
	SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);

	// Set the window title before setting the display mode, so that you never
	// see that SDL_display title
	SDL_WM_SetCaption("Egoboo Model Viewer", NULL);

	flags = SDL_OPENGL;
	if(fullscreen) flags |= SDL_FULLSCREEN;

	m_surface = SDL_SetVideoMode(width, height, bitDepth, flags);
	if(!m_surface)
	{
		return false;
	}

	// SDL generates a couple of mouse movements right when  it starts up, 
	// because it moves the cursor to the middle of the screen.  Eat those here,
	// so that they don't cause any jumping once the game starts
	while(SDL_PollEvent(&e));

	return true;
}

void DisplayManager::closeDisplay()
{
	if (m_surface != NULL)
	{
		SDL_QuitSubSystem(SDL_INIT_VIDEO);
		m_surface = NULL;
	}
}

int DisplayManager::displayWidth() const
{
	return (m_surface != NULL) ? m_surface->w : 0;
}

int DisplayManager::displayHeight() const
{
	return (m_surface != NULL) ? m_surface->h : 0;
}
