/* Zodiac - LevelGeometry.cpp
 */

#include "LevelGeometry.h"
#include <string.h>
#include <stdio.h>

LevelGeometry::LevelGeometry()
{
	m_mpdMesh = NULL;
	memset(m_mpdTextures, 0, sizeof(Texture*) * MPD_NumTextures);
	memset(m_fanTypes, 0, sizeof(MpdFanType) * MPD_MaxFanTypes);
}

LevelGeometry::~LevelGeometry()
{
	freeMesh();
	freeTextures();
}

bool LevelGeometry::loadFanTypes(const char *fileName)
{
	// Ignore garbage input
	if (!fileName || !fileName[0]) return false;

	resetFanTypes();
	return MpdMesh::loadFanTypes(fileName, m_fanTypes);
}

bool LevelGeometry::loadLevel(const char *levelDir)
{
	// No matter what, calling loadLevelGeometry is going to clear out the currently 
	// loaded LevelGeometry.
	freeMesh();
	freeTextures();

	// Bail on garbage input. 
	if (!levelDir || !levelDir[0]) return false;

	char buffer[256];
	_snprintf(buffer, 256, "modules/%s/mesh.mpd", levelDir);
	m_mpdMesh = MpdMesh::loadFromFile(buffer);
	if (!m_mpdMesh)
	{
		return false;
	}

	for (int i = 0;i < MPD_NumTextures; i++)
	{
		_snprintf(buffer, 256, "modules/%s/tris%d.bmp", levelDir, i);
		m_mpdTextures[i] = Texture::loadFromFile(buffer);
		if (!m_mpdTextures[i])
		{
			return false;
		}
	}

	return true;
}

void LevelGeometry::resetFanTypes()
{
	memset(m_fanTypes, 0, sizeof(MpdFanType) * MPD_MaxFanTypes);
}

void LevelGeometry::freeMesh()
{
	if (m_mpdMesh != NULL)
	{
		delete m_mpdMesh;
		m_mpdMesh = NULL;
	}
}

void LevelGeometry::freeTextures()
{
	for (int i = 0;i < MPD_NumTextures; i++)
	{
		if (m_mpdTextures[i] != NULL)
		{
			delete m_mpdTextures[i];
			m_mpdTextures[i] = NULL;
		}
	}
}

const Texture* LevelGeometry::getTexture(int index) const
{
	if (index >= 0 && index < MPD_NumTextures)
	{
		return m_mpdTextures[index];
	}
	return NULL;
}
