/* Zodiac - MpdMesh.h
 * Egoboo MPD mesh loader.
 */

#ifndef Zodiac_MpdMesh
#define Zodiac_MpdMesh

enum MpdConstant
{
    MPD_FileID		  = 0x4470614d,	// The string... MapD
    MPD_NumTextures	  = 4,
    MPD_MaxCommands	  = 4,
    MPD_MaxCommandEntries = 32,
    MPD_MaxVertices	  = 16,
    MPD_MaxFanTypes	  = 64,
    MPD_MaxTextureTiles	  = 256,
    MPD_MaxFans		  = (512*512),
    MPD_MaxSizeY	  = 1024,
    MPD_FanOff		  = 0xffff,	 // an empty fan
};

struct MpdVertex
{
    float x, y, z;		// position data
    unsigned int color;	// packed rgb color
    float s, t;			// texture coordinates
    unsigned char ambient, light;	// Lighting data
    unsigned char reserved[6];		// Pack structure to 32 bytes
};

struct MpdFanType
{
    int numCommands;
    int numCommandEntries[MPD_MaxCommands];
    int numVertices;
    unsigned short vertexIndices[MPD_MaxCommandEntries];
    float texCoords[MPD_MaxVertices][2];
};

struct MpdFan
{
    unsigned char type;		// Index into FanType array
    unsigned char flags;	// Fan property flags
    int	textureTile;		// Tile texture the fan uses
    unsigned char twist;	// ?
    unsigned int firstVertex;	// This fans first vertex in the mesh's vertex array
};

class MpdMesh
{
 private:
    int m_numVertices;
    int m_fansWide;
    int m_fansHigh;
    int m_waterShift;
    float m_width, m_height;

    MpdVertex *m_vertices;
    MpdFan *m_fans;

    bool m_prepared;

    MpdMesh();

 public:
    virtual ~MpdMesh();

    static MpdMesh *loadFromFile(const char *fileName);
    static bool     loadFanTypes(const char *fileName, MpdFanType *fanTypes);

    // prepareVertices calculates the "firstVertex" member of each vertex.  It needs
    // to be called before you can draw the mesh, or call getHeight
    void prepareVertices(const MpdFanType *fanTypes);

    // Return the mesh's height at the given point. 
    float getHeight(float x, float y) const;

    // Retrieve info about the mesh
    bool prepared() const { return m_prepared; }
    int numVertices() const { return m_numVertices; }
    int fansWide() const { return m_fansWide; }
    int fansHigh() const { return m_fansHigh; }
    float width() const { return m_width; }
    float height() const { return m_height; }
    int numFans() const { return (m_fansWide * m_fansHigh); }

    const MpdVertex* getVertices() const { return m_vertices; }
    const MpdFan* getFan(int index) const;
};

#endif // include guard
