/* Zodiac - System_Win32.cpp
 * Win32 implementation of the System interface.
 */

#include "Common.h"
#define WIN32_LEAN_AND_MEAN
#include <windows.h>

using namespace std;

list<string> sys_listFiles(const char *fileSpec)
{
	HANDLE hList;
	WIN32_FIND_DATA fileData;
	list<string> fileMatches;

	// Start looking through the directory
	hList = FindFirstFile(fileSpec, &fileData);
	if (hList == INVALID_HANDLE_VALUE)
	{
		FindClose(hList);
		return fileMatches;
	}

	bool finished = false;
	while (!finished)
	{
		// Ignore directories and files that start w/ a dot
		if (fileData.cFileName && fileData.cFileName[0] != '.')
		{
			fileMatches.push_back(string(fileData.cFileName));
		}

		if (!FindNextFile(hList, &fileData))
		{
			if (GetLastError() == ERROR_NO_MORE_FILES)
			{
				finished = true;
			}
		}
	}

	FindClose(hList);
	return fileMatches;
}

// Timing functionality
double win32_secondsPerTick = 0;

void sys_initialize()
{
	LARGE_INTEGER frequency;

	QueryPerformanceFrequency(&frequency);
	win32_secondsPerTick = 1.0 / frequency.QuadPart;
}

void sys_shutdown()
{}

double sys_getTime()
{
	LARGE_INTEGER time;

	QueryPerformanceCounter(&time);
	return time.QuadPart * win32_secondsPerTick;
}
